""" Handler to get and log user's contact in database
> Checks to see if the user is violating the contact rules
> > If user is eligble for contact, set the email in the database
> > If the user has violated one of the contact rules:
> > > Give a status code and return the expected error messaging
> > > TODO LOG INSTANCE IN ERROR HANDLER
"""
from email_controller.record_validation import RecordValidation
from email_controller.smtp_controller import GMailController
from utils_package.data_controller.scripts.email_controller.email_audit_queries import AuditWriter
from utils_package.data_controller.json_config import JSONConfig
from datetime import datetime
from utils_package.py_utils.logger import logger


class ContactHandler(object):
    """ Handles contact form entries """
    PRIMARY_ADDRESS = 'jamberin@gmail.com'

    def __init__(self):
        """ Initialize class variables """
        self.record_validation = RecordValidation()
        self.audit_writer = AuditWriter()
        self.config = JSONConfig()
        self.gmail = GMailController(self.config.get_smtp_dict('primary_gmail'))

    def contact_form_entry(self, name, email, message):
        """
        For contact form entries, validate the email, handle responses, and send the email
        :param name:
        :param email:
        :param message:
        :return: Status code, display message, display validation
        """
        # Validate that the email address associated is eligible for contact
        response = self.record_validation.validate_rule_for_contact(email)
        if response['validation'] is True:
            error_message = self.__record_validation_error_handler(response)
            error_status = 429
            return error_status, error_message['display_message'], error_message
        else:
            chk = self.send_contact_form_email(name, email, message)
            if chk is True:
                record_dict = {
                    'strname': name,
                    'stremailaddress': email,
                    'strmessage': message
                }
                response = self.audit_writer.insert_new_record(record_dict)
                assert response[0] == 1, 'Record was not created, ensure database working: ' \
                                         '[name: %s, email: %s, message: %s]' % (name, email, message)
                response_status = 201
                response_message = {
                    'violation_code': [0],
                    'violation_reasons': [None],
                    'display_message': 'Your message has been sent! I will be in contact shortly.'
                }
            else:
                logger.error('Issues sending message')
                return 500, 'Internal Server Error', email
            return response_status, response_message['display_message'], response_message

    def send_contact_form_email(self, name, email_address, message):
        """
        Sends a simple plaintext email regarding contact
        :param name: Name from contact
        :param email_address: Email address from contact
        :param message: Message that is to be sent
        :return: Success or failure message
        """
        date_of_contact = datetime.strftime(datetime.now(), '%x')
        email_message = """
        Timestamp: %s
        Name: %s
        eMail Address: %s
        Message: %s
        """ % (date_of_contact, name, email_address, message)
        subject = 'Contact from %s | %s' % (name, date_of_contact)
        response = self.gmail.attempt_send_message(email_message, self.PRIMARY_ADDRESS, subject)
        return response

    @staticmethod
    def __record_validation_error_handler(error_response):
        """
        Builds the error response to be sent back to the front end
        :param error_response: Depending on the error_response content, the appropriate error message is sent
        :return: Error response reason and message
        """
        # Build the error message and code to be sent to the front end
        message = {
            'violation_code': [],
            'violation_reasons': [],
            'display_message': 'It looks like you have tried to contact too many times. '
        }

        # For the given violation reasons
        if 'DAY' in error_response['val_reason']:
            message['violation_code'].append(1)
            message['violation_reasons'].append('Daily contact violation')
            message['display_message'] = message['display_message'] + 'Over three times in the last day. '
        if 'WEEK' in error_response['val_reason']:
            message['violation_code'].append(2)
            message['violation_reasons'].append('Weekly contact violation')
            message['display_message'] = message['display_message'] + 'More than five times in the past week. '
        if 'MONTH' in error_response['val_reason']:
            message['violation_code'].append(3)
            message['violation_reasons'].append('Monthly contact violation')
            message['display_message'] = message['display_message'] + 'Over twenty times in the last month. '
        if 'ANNUAL' in error_response['val_reason']:
            message['violation_code'].append(4)
            message['violation_reasons'].append('Annual contact violation')
            message['display_message'] = message['display_message'] + 'More than forty times in the year month. '

        return message



""" Validation that a certain email address hasn't violated usage rules
Rules:
0. Less than 3 contacts within the past day
1. Less than 5 contacts within the past week
2. Less than 20 contacts within the past month
3. Less than 40 contacts within the past year
"""
from utils_package.data_controller.scripts.email_controller.email_audit_queries import AuditReader
from utils_package.data_controller.scripts.email_controller.contact_violations_queries import ViolationsWriter
from utils_package.py_utils.logger import logger


class RecordValidation(object):

    def __init__(self):
        """ Initialize Class Variables """
        # Queries
        self.audit_reader = AuditReader()
        self.violations_writer = ViolationsWriter()

    def validate_rule_for_contact(self, contact_email):
        """
        Verify that the email is eligible for another email to be sent
        :param contact_email: String of the email address
        :return: Dict response [boolean check, list rule broken (if applicable)]
        """
        # Build response dictionary
        response = {
            'validation': False,
            'val_reason': []
        }

        # Violation dict for tracking violations
        violation_dict = {
            'strViolationType': None,
            'strViolationAddress': contact_email
        }

        # Collect all contact counts
        day_contact = self.audit_reader.get_record_for_email_within_past_days(contact_email, 0)
        logger.info('Day Count: %s' % str(day_contact))
        week_contact = self.audit_reader.get_record_for_email_within_past_days(contact_email, 6)
        logger.info('Week Count: %s' % str(week_contact))
        month_contact = self.audit_reader.get_record_for_email_within_past_days(contact_email, 29)
        logger.info('Month Count: %s' % str(month_contact))
        year_contact = self.audit_reader.get_record_for_email_within_past_days(contact_email, 364)
        logger.info('Year Count: %s' % str(year_contact))

        # Verify all contact counts
        if day_contact[0] >= 3:
            response['validation'] = True
            response['val_reason'].append('DAY')
            violation_dict['strViolationType'] = 'DAY'
            logger.warning('Rule violation: Daily Contact limit reacheacd: %s' % contact_email)
        if week_contact[0] >= 5:
            response['validation'] = True
            response['val_reason'].append('WEEK')
            violation_dict['strViolationType'] = 'WEEK'
            logger.warning('Rule violation: Weekly Contact limit reacheacd: %s' % contact_email)
        if month_contact[0] >= 20:
            response['validation'] = True
            response['val_reason'].append('MONTH')
            violation_dict['strViolationType'] = 'MONTH'
            logger.warning('Rule violation: Monthly Contact limit reacheacd: %s' % contact_email)
        if year_contact[0] >= 40:
            response['validation'] = True
            response['val_reason'].append('ANNUAL')
            violation_dict['strViolationType'] = 'ANNUAL'
            logger.warning('Rule violation: Yearly Contact limit reacheacd: %s' % contact_email)

        # Write validation error if applicable
        if response['validation']:
            assert violation_dict['strViolationType'] is not None
            self.violations_writer.insert_new_record(violation_dict)

        # TODO BUILD QUEUE TO HANDLE RESPONSES TO USERS
        # Include logic to send an email to the user denoting response
        # Find way to handle contact violation management, perhaps IP blocking
        # For now, just generate way to send canned response for good responses only

        return response



""" Unit tests for the email email_controller application
> TODO Add tests for rules violations
"""
import random
from unittest import TestCase
from email_controller.contact_handler import ContactHandler
from utils_package.data_controller.scripts.email_controller.email_audit_queries import AuditReader


class TestEmailController(TestCase):
    """ Send a sample email """

    def setUp(self):
        """ Set up class variables """
        self.contact = ContactHandler()
        self.audit_reader = AuditReader()

    def test_send_email_no_violation(self):
        """ Send a sample email like what would come from the contact form
        1. Set up test data
        2. Send data to contact form
        3. Confirm contact form record stored
        """
        # 1. Set up test data
        name = 'John Denver'
        email = 'john.denver%s@website.net' % str(random.randint(0000, 9999))
        message = 'Test simple plain text message'

        # 2. Send data to contact form
        status_code, display_message, response = self.contact.contact_form_entry(name, email, message)
        self.assertEqual(201, status_code)
        self.assertEqual('Your message has been sent! I will be in contact shortly.', display_message)
        self.assertIsNotNone(response)

        # 3. Confirm contact form record stored
        response = self.audit_reader.get_all_record_for_email(email)
        self.assertIsNotNone(response)

    def test_violation(self):
        """ Send more than 3 emails in one day
        1. Set up test data
        2. Send data to contact form
        3. Confirm contact form records stored
        4. Validate the error response on the last run
        """
        # 1. Set up test data
        name = 'John Denver'
        email = 'john.denver%s@website.net' % str(random.randint(0000, 9999))
        message = 'Test simple plain text message'

        # 2. Send data to contact form
        r = range(3)
        for _ in r:
            status_code, display_message, response = self.contact.contact_form_entry(name, email, message +
                                                                                     ' %s' % str(_))
            self.assertEqual(201, status_code)
            self.assertEqual('Your message has been sent! I will be in contact shortly.', display_message)
            self.assertIsNotNone(response)

        # 3. Confirm contact form records stored
        response = self.audit_reader.get_all_record_for_email(email)
        self.assertIsNotNone(response)

        # 4. Validate the error response on the last run
        status_code, display_message, response = self.contact.contact_form_entry(name, email, message)
        self.assertEqual(429, status_code)
        self.assertEqual('It looks like you have tried to contact too many times. Over three times in the last day. ',
                         display_message)
        self.assertIsNotNone(response)



""" Unit tests for the response handler application """
from email_controller.response_handler import ResponseHandler
from unittest import TestCase


class TestResponseHandler(TestCase):
    """ Unit tests for response handler """

    def setUp(self):
        """ Set up class variables """
        self.response_handler = ResponseHandler()

    def test_send_email(self):
        """ Test to send the standard response email
        1. Set up test data
        2. Send email
        3. Verify response
        """
        # 1. Set up test data
        email_address = 'jamberin@gmail.com'

        # 2. Send email
        response = self.response_handler.send_response_email(email_address)

        # 3. Verify response
        self.assertTrue(response)



""" Primary email_controller for SMTP control
> Will handlge the default SMTP control to format HTML and send emails
> Configured for gmail only currently
"""
from smtplib import SMTPDataError, SMTPSenderRefused, SMTPRecipientsRefused, SMTPHeloError, SMTP
from utils_package.py_utils.logger import logger
from utils_package.data_controller.json_config import JSONConfig
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
import codecs

# Declarations
json_config = JSONConfig()


def log_email_payload(login_dict, message, recipient, subject):
    """
    Reusable method to log email payload
    :param login_dict:
    :param message:
    :param recipient:
    :param subject:
    """
    logger.debug('Login Dict: [user: %s, pass: %s]' % (login_dict['user'], login_dict['pass']))
    logger.debug('Message: %s' % str(message))
    logger.debug('Subject: %s' % subject)
    logger.debug('Recipient: %s' % recipient)


def get_html_template(template_name):
    """
    Return the HTML content of the template file
    :param template_name: Location of the file within the email templates directory
    :return: HTML template
    """
    template_dir = json_config.get_email_controller_config('directories')['templates']
    template_location = template_dir + template_name
    file = codecs.open(template_location, 'r')
    f = file.read()
    file.close()
    return f


class GMailController(object):
    """ Main email_controller for SMTP """

    def __init__(self, login_dict):
        """
        Initialize class variables
        :param login_dict: login_dict: Dictionary of login credentials [user, pass]
        """
        self.to_address = login_dict['user']
        server = SMTP('smtp.gmail.com', 587)
        server.starttls()
        server.login(login_dict['user'], login_dict['pass'])
        self.server = server

    def attempt_send_message(self, message, recipient, subject, message_type='text'):
        """
        Attempts to send the message
        :param
        :param message: HTML message object
        :param recipient: Recipient of the email
        :param subject: Subject line of the email
        :param message_type: Either text or html
        :return: Success or failure messages
        """
        chk = True
        logger.info('Starting email send attempt')
        html_msg = MIMEMultipart('alternative')
        html_msg['Subject'] = subject
        html_msg['To'] = recipient

        if message_type.lower() == 'text':
            message_builder = MIMEText(message, 'plain')
        elif message_type.lower() == 'html':
            message_builder = MIMEText(str(message), 'html')
        else:
            logger.error('Message type not recognized')
            raise Exception('Message type not recognized')

        html_msg.attach(message_builder)

        try:
            self.server.sendmail(self.to_address, recipient, html_msg.as_string())
        except SMTPDataError or SMTPSenderRefused or SMTPRecipientsRefused or SMTPHeloError:
            logger.error('Issue with sending email')
            chk = False
        logger.info('Successfully sent email')
        return chk



""" Handler to respond to and log after a user's contact
> Sends a response email to a user
"""
from email_controller.smtp_controller import GMailController, get_html_template
from utils_package.data_controller.json_config import JSONConfig
from utils_package.py_utils import primary_utils


class ResponseHandler(object):
    """ Handles responses to users """

    def __init__(self):
        """ Initialize class variables """
        self.config = JSONConfig()
        self.gmail = GMailController(self.config.get_smtp_dict('primary_gmail'))
        self.utils = primary_utils

    def send_response_email(self, email_address):
        """
        Sends the email template
        :param email_address:
        :return:
        """
        template = get_html_template('contact_response.html')
        subject = 'beringersolutions.com | Thanks for reaching out!'
        response = self.gmail.attempt_send_message(template, email_address, subject, message_type='html')
        return response



import setuptools

with open('README.md', 'r') as read_me:
    description = read_me.read()

setuptools.setup(
    name='email_controller',
    version='0.0.6',
    author='James Beringer',
    author_email='jamberin@gmail.com',
    description='Package for just basic SMTP email sending',
    long_description=description,
    long_description_content_type='text/markdown',
    url='https://github.com/jamberin/email_controller',
    packages=setuptools.find_packages(),
    classifiers=[
        'Programming Language :: Python :: 3.7',
        'Development Status :: 2 - Pre-Alpha',
        'License :: Public Domain'
    ],
    include_package_data=True
)
